@@ -29,7 +29,8 @@ from xml.etree.ElementTree import Element, SubElement, tostring, parse
 from pitivi.reflect import qual, namedAny
 from pitivi.factories.base import SourceFactory
 from pitivi.factories.file import FileSourceFactory
-from pitivi.timeline.track import Track
+from pitivi.factories.operation import EffectFactory
+from pitivi.timeline.track import Track, TrackEffect
 from pitivi.timeline.timeline import TimelineObject
 from pitivi.formatters.base import Formatter, FormatterError
 from pitivi.utils import get_filesystem_encoding
@@ -70,8 +71,8 @@ class ElementTreeFormatter(Formatter):
     _element_id = 0
     _our_properties = ["id", "type"]
 
-    def __init__(self, *args, **kwargs):
-        Formatter.__init__(self, *args, **kwargs)
+    def __init__(self, avalaible_effects, *args, **kwargs):
+        Formatter.__init__(self, avalaible_effects, *args, **kwargs)
         self.factoriesnode = None
         self.timelinenode = None
         self._settingsnode = None
@@ -91,9 +92,18 @@ class ElementTreeFormatter(Formatter):
             yield name, value
 
     def _parsePropertyValue(self, value):
-        # nothing to read here, move along
-        # edward: argh, I went past there, what shall I do now ?
-        return gst.Caps("meh, name=%s" % value)[0]["name"]
+        # We treat the GEnum values differently because they are actually ints.
+        if "(GEnum)" in value:
+            return int(value.split("(GEnum)")[1])
+        # We treat the guint64 values differently because when we serialize
+        # a long integer, the serialization is, for example, "(guint64) 4L",
+        # and gst.Caps() fails to parse it, because it has "L" at the end.
+        if "(guint64)" in value:
+            value = value.rstrip('lL')
+        # TODO: Use what gst.Caps() uses to parse a property value.
+        caps = gst.Caps("structure1, property1=%s;" % value)
+        structure = caps[0]
+        return structure["property1"]
 
     def _saveStream(self, stream):
         element = Element("stream")
@@ -178,15 +188,17 @@ class ElementTreeFormatter(Formatter):
         factory.duration = long(element.attrib["duration"])
         factory.default_duration = long(element.attrib["default_duration"])
 
-        input_streams = element.find("input-streams") or []
-        for stream_element in input_streams:
-            stream = self._loadStream(stream_element)
-            factory.addInputStream(stream)
+        input_streams = element.find("input-streams")
+        if input_streams is not None:
+            for stream_element in input_streams:
+                stream = self._loadStream(stream_element)
+                factory.addInputStream(stream)
 
         output_streams = element.find("output-streams")
-        for stream_element in output_streams:
-            stream = self._loadStream(stream_element)
-            factory.addOutputStream(stream)
+        if output_streams is not None:
+            for stream_element in output_streams:
+                stream = self._loadStream(stream_element)
+                factory.addOutputStream(stream)
 
         if filename is not None:
             filename1 = self.validateSourceURI(filename, factory)
@@ -242,30 +254,34 @@ class ElementTreeFormatter(Formatter):
         self._sources = res
         return res
 
-    def _serializeDict(self, element, dict):
-        for a, b in dict.iteritems():
-            print a, b, type(b)
-            if isinstance(b, str):
-                element.attrib[a] = b
-            elif isinstance(b, bool):
-                element.attrib[a] = "(boolean) %r" % b
-            elif isinstance(b, float):
-                element.attrib[a] = "(float) %r" % b
+    def _serializeDict(self, element, values_dict):
+        """Serialize the specified dict into the specified Element instance."""
+        for key, value in values_dict.iteritems():
+            if isinstance(value, str):
+                # TODO: If this starts with "(<type>)", or if it contains ";",
+                # the deserialization might fail.
+                serialized_value = value
+            elif isinstance(value, bool):
+                serialized_value = "(boolean) %r" % value
+            elif isinstance(value, float):
+                serialized_value = "(float) %r" % value
             else:
-                element.attrib[a] = "(guint64) %r" % b
-
+                serialized_value = "(guint64) %r" % value
+            element.attrib[key] = serialized_value
 
     def _deserializeDict(self, element):
-        d = {}
-        for a, b in element.attrib.iteritems():
-            if len(b):
-                d[a] = self._parsePropertyValue(b)
-        return d
+        """Get the specified Element as a deserialized dict."""
+        values_dict = {}
+        for name, value_string in element.attrib.iteritems():
+            if value_string:
+                values_dict[name] = self._parsePropertyValue(value_string)
+        return values_dict
 
     def _saveProjectSettings(self, settings):
         element = Element('export-settings')
         element.attrib["videowidth"] = str(int(settings.videowidth))
         element.attrib["videoheight"] = str(int(settings.videoheight))
+        element.attrib["render-scale"] = str(int(settings.render_scale))
         element.attrib["videorate-num"] = str(int(settings.videorate.num))
         element.attrib["videorate-denom"] = str(int(settings.videorate.denom))
         element.attrib["videopar-num"] = str(int(settings.videopar.num))
@@ -294,6 +310,8 @@ class ElementTreeFormatter(Formatter):
         settings = ExportSettings()
         settings.videowidth = int(element.attrib["videowidth"])
         settings.videoheight = int(element.attrib["videoheight"])
+        if "render-scale" in element.attrib:
+            settings.render_scale = int(element.attrib["render-scale"])
         settings.videorate = gst.Fraction(int(element.attrib["videorate-num"]),
                                          int(element.attrib["videorate-denom"]))
         settings.videopar = gst.Fraction(int(element.attrib["videopar-num"]),
@@ -301,8 +319,8 @@ class ElementTreeFormatter(Formatter):
         settings.audiochannels = int(element.attrib["audiochannels"])
         settings.audiorate = int(element.attrib["audiorate"])
         settings.audiodepth = int(element.attrib["audiodepth"])
-        settings.aencoder = element.attrib["aencoder"] or None
-        settings.vencoder = element.attrib["vencoder"] or None
+        settings.aencoder = element.attrib["aencoder"]
+        settings.vencoder = element.attrib["vencoder"]
         settings.muxer = element.attrib["muxer"]
 
         sett = element.find("container-settings")
@@ -327,11 +345,46 @@ class ElementTreeFormatter(Formatter):
                     str("(gint64)%s" % getattr(track_object, attribute))
 
         element.attrib["priority"] = "(int)%s" % track_object.priority
+        element.attrib["active"] = "(bool)%s" % track_object.active
 
-        factory_ref = \
-                self._saveFactoryRef(track_object.factory)
+        if not isinstance(track_object.factory, EffectFactory):
+            self._saveSourceTrackObject(track_object, element)
+        else:
+            self._saveTrackEffect(track_object, element)
+
+        self._context.track_objects[track_object] = element
+
+        return element
+
+    def _saveTrackEffect(self, track_object, element):
+        effect_element = Element("effect")
+        element.append(effect_element)
+
+        factory_element = Element("factory")
+        factory_element.attrib["name"] = track_object.factory.name
+        effect_element.append(factory_element)
+
+        self._saveEffectProperties(track_object, effect_element)
+
+    def _saveEffectProperties(self, track_object, effect_element):
+        effect_properties = Element("gst-element-properties")
+        effect = track_object.getElement()
+        properties = gobject.list_properties(effect)
+        for prop in properties:
+            if prop.flags & gobject.PARAM_READABLE:
+                type_name = str(gobject.type_name(prop.value_type.fundamental))
+                #FIXME we just take the int equivalent to the GEnum, how should it be handled?
+                if type_name == "GEnum":
+                    value = str(effect.get_property(prop.name).__int__())
+                else:
+                    value = str(effect.get_property(prop.name))
+                effect_properties.attrib[prop.name] = '(' + type_name + ')' + value
+        effect_element.append(effect_properties)
+
+
+    def _saveSourceTrackObject(self, track_object, element):
+        factory_ref = self._saveFactoryRef(track_object.factory)
         stream_ref = self._saveStreamRef(track_object.stream)
-
         element.append(factory_ref)
         element.append(stream_ref)
         interpolators = track_object.getInterpolators()
@@ -340,14 +393,48 @@ class ElementTreeFormatter(Formatter):
             curves.append(self._saveInterpolator(interpolator, property))
         element.append(curves)
 
-        self._context.track_objects[track_object] = element
-
-        return element
-
     def _loadTrackObject(self, track, element):
         self.debug("%r", element)
         klass = namedAny(element.attrib["type"])
+        if klass is TrackEffect:
+            track_object = self._loadEffectTrackObject(element, klass, track)
+        else:
+            track_object = self._loadSourceTrackObject(element, klass, track)
+        return track_object
 
+    def _loadEffectTrackObject(self, element, klass, track):
+        effect_element = element.find('effect')
+        factory_name = effect_element.find('factory').attrib['name']
+        properties_elem = effect_element.find('gst-element-properties')
+        try:
+            factory = self.avalaible_effects.getFactoryFromName(factory_name)
+        except KeyError:
+            #Find a way to figure out how we could install the missing effect...
+            raise FormatterError(_("The project contains effects that are not\
+                                   avalaibe on the system. It can't be loaded"))
+
+        input_stream = factory.getInputStreams()
+        if not input_stream:
+            raise FormatterError("cant find effect factory input stream")
+        input_stream = input_stream[0]
+
+        track_object = klass(factory, input_stream)
+
+        track.addTrackObject(track_object)
+
+        for name, value_string in self._filterElementProperties(element):
+            value = self._parsePropertyValue(value_string)
+            setattr(track_object, name, value)
+
+        effect_gst_element = track_object.getElement()
+        for name, value in properties_elem.attrib.iteritems():
+            value = self._parsePropertyValue(value)
+            effect_gst_element.set_property(name, value)
+
+        self._context.track_objects[element.attrib["id"]] = track_object
+        return track_object
+
+    def _loadSourceTrackObject(self, element, klass, track):
         factory_ref = element.find("factory-ref")
         factory = self._loadFactoryRef(factory_ref)
 
@@ -360,7 +447,7 @@ class ElementTreeFormatter(Formatter):
             setattr(track_object, name, value)
         track.addTrackObject(track_object)
         curves_element = element.find("curves")
-        if curves_element:
+        if curves_element is not None:
             for curve in curves_element.getchildren():
                 self._loadInterpolator(curve, track_object)
 
@@ -402,7 +489,7 @@ class ElementTreeFormatter(Formatter):
         interpolator.start.mode = int(start.attrib["mode"])
 
         for kf in element.getiterator("keyframe"):
-            interpolator.newKeyframe(long(kf.attrib["time"]), 
+            interpolator.newKeyframe(long(kf.attrib["time"]),
                 value=self._parsePropertyValue(kf.attrib["value"]),
                 mode=int(kf.attrib["mode"]))
         end = element.find("end")
@@ -634,6 +721,7 @@ class ElementTreeFormatter(Formatter):
             return
 
         uris = [source.uri for source in sources]
+        project.sources.nb_file_to_import = len(uris)
         discoverer = project.sources.discoverer
         discoverer.connect("discovery-done", self._discovererDiscoveryDoneCb,
                 project, sources, uris, closure)
