@@ -25,18 +25,25 @@ Settings
 """
 
 import os
+import platform
 import gst
 from ConfigParser import SafeConfigParser, ParsingError
+import xdg.BaseDirectory as xdg_dirs  # Freedesktop directories spec
 
 from gettext import gettext as _
+from gettext import ngettext
 
 from pitivi.signalinterface import Signallable
-from pitivi.encode import available_muxers, available_video_encoders, \
-     available_audio_encoders, available_combinations, \
+from pitivi.encode import available_combinations, \
      get_compatible_sink_caps
 from pitivi.stream import get_stream_for_caps
 from pitivi.log.loggable import Loggable
 
+if platform.system() == 'Windows':
+    HOME = 'USERPROFILE'
+else:
+    HOME = 'HOME'
+
 def get_bool_env(var):
     value = os.getenv(var)
     if not value:
@@ -51,7 +58,7 @@ def get_bool_env(var):
 
 def get_env_by_type(type_, var):
     """
-    Returns the environement variable.
+    Returns the environment variable.
 
     @arg type_: The type of the variable
     @type type_: C{type}
@@ -60,6 +67,8 @@ def get_env_by_type(type_, var):
     @returns: The content of the environment variable, or C{None} if it doesn't
     exist.
     """
+    if var is None:
+        return None
     if type_ == bool:
         return get_bool_env(var)
     else:
@@ -80,7 +89,7 @@ def get_dir(path, autocreate=True):
     return path
 
 def get_dirs(glob):
-    return [d for d in glob.split(os.path.pathsep) if os.path.exists(dir)]
+    return [d for d in glob.split(os.path.pathsep) if os.path.exists(d)]
 
 def get_env_dir(var, default, autocreate=True):
     return get_dir(get_env_default(var, default))
@@ -89,24 +98,19 @@ def get_env_dirs(var, default):
     return get_dirs(get_env_default(var, default))
 
 def xdg_config_home(autocreate=True):
-    return get_dir(get_env_default("XDG_CONFIG_HOME",
-        os.path.join(os.getenv("HOME"), ".config")), autocreate)
+    return get_dir(xdg_dirs.xdg_config_home, autocreate)
 
 def xdg_data_home(autocreate=True):
-    return get_dir(get_env_default("XDG_DATA_HOME",
-        os.path.join(os.getenv("HOME"), ".local", "share")), autocreate)
+    return get_dir(xdg_dirs.xdg_data_home, autocreate)
 
 def xdg_cache_home(autocreate=True):
-    return get_dir(get_env_default("XDG_CACHE_HOME",
-        os.path.join(os.getenv("HOME"), ".cache")), autocreate)
+    return get_dir(xdg_dirs.xdg_cache_home, autocreate)
 
 def xdg_data_dirs():
-    return get_env_dirs("XDG_DATA_DIRS",
-        os.path.pathsep.join((os.path.join("usr", "local", "share"),
-            os.path.join("usr", "share"))))
+    return get_env_dirs(xdg_dirs.xdg_data_dirs)
 
 def xdg_config_dirs():
-    return get_env_dirs("XDG_CONFIG_DIRS", "/etc/xdg")
+    return get_env_dirs(xdg_dirs.xdg_config_dirs)
 
 class ConfigError(Exception):
     pass
@@ -453,25 +457,30 @@ class ExportSettings(Signallable, Loggable):
     # TODO : Add PAR/DAR for video
     # TODO : switch to using GstFraction internally where appliable
 
+
+    # The following dependant attributes caches are common to all instances!
+    # A (muxer -> containersettings) map.
+    _containersettings_cache = {}
+    # A (vencoder -> vcodecsettings) map.
+    _vcodecsettings_cache = {}
+    # A (aencoder -> acodecsettings) map.
+    _acodecsettings_cache = {}
+
+    muxers, aencoders, vencoders = available_combinations()
+
     def __init__(self, **unused_kw):
         Loggable.__init__(self)
         self.videowidth = 720
         self.videoheight = 576
+        self.render_scale = 100
         self.videorate = gst.Fraction(25, 1)
-        self.videopar = gst.Fraction(1, 1)
+        self.videopar = gst.Fraction(16, 15)
         self.audiochannels = 2
         self.audiorate = 44100
         self.audiodepth = 16
         self.vencoder = "theoraenc"
         self.aencoder = "vorbisenc"
         self.muxer = "oggmux"
-        self.containersettings = {}
-        self.acodecsettings = {}
-        self.vcodecsettings = {}
-        self.muxers = available_muxers()
-        self.vencoders = available_video_encoders()
-        self.aencoders = available_audio_encoders()
-        self.muxers = available_combinations(self.muxers, self.vencoders, self.aencoders)
 
     def copy(self):
         ret = ExportSettings()
@@ -490,6 +499,9 @@ class ExportSettings(Signallable, Loggable):
         ret.vcodecsettings = dict(self.vcodecsettings)
         return ret
 
+    def getDAR(self):
+        return gst.Fraction(self.videowidth, self.videoheight) * self.videopar
+
     def __str__(self):
         msg = _("Export Settings\n")
         msg += _("Video: ") + str(self.videowidth) + " " + str(self.videoheight) +\
@@ -501,37 +513,31 @@ class ExportSettings(Signallable, Loggable):
         msg += _("\nMuxer: ") + str(self.muxer) + " " + str(self.containersettings)
         return msg
 
-    def getVideoCaps(self):
+    def getVideoWidthAndHeight(self, render=False):
+        """ Returns the video width and height as a tuple
+
+        @param render: Whether to apply self.render_scale to the returned
+        values.
+        @type render: bool
+        """
+        if render:
+            scale = self.render_scale
+        else:
+            scale = 100
+        return self.videowidth * scale / 100, self.videoheight * scale / 100
+
+    def getVideoCaps(self, render=False):
         """ Returns the GstCaps corresponding to the video settings """
-        astr = "width=%d,height=%d,pixel-aspect-ratio=%d/%d,framerate=%d/%d" % (self.videowidth,
-                                                                                self.videoheight,
-                                                                                self.videopar.num,
-                                                                                self.videopar.denom,
-                                                                                self.videorate.num,
-                                                                                self.videorate.denom)
+        videowidth, videoheight = self.getVideoWidthAndHeight(render=render)
+        astr = "width=%d,height=%d,pixel-aspect-ratio=%d/%d,framerate=%d/%d" % (
+                videowidth, videoheight,
+                self.videopar.num, self.videopar.denom,
+                self.videorate.num, self.videorate.denom)
         vcaps = gst.caps_from_string("video/x-raw-yuv,%s;video/x-raw-rgb,%s" % (astr, astr))
         if self.vencoder:
             return get_compatible_sink_caps(self.vencoder, vcaps)
         return vcaps
 
-    def getVideoDescription(self):
-        """ Returns a human-readable markup-ed string describing the video properties """
-        res = "%d x %d <i>pixels</i> at %.2f <i>fps</i>" % (self.videowidth,
-                self.videoheight, float(self.videorate))
-        if self.vencoder is not None:
-            res += " <i>(%s)</i>" % self.vencoder
-
-        return res
-
-    def getAudioDescription(self):
-        """ Returns a human-readable markup-ed string describing the audio properties """
-        res = "%d channels at %d <i>Hz</i> (%d <i>bits</i>)" % \
-                (self.audiochannels, self.audiorate, self.audiodepth)
-        if self.aencoder is not None:
-            res += " <i>(%s)</i>" % self.aencoder
-
-        return res
-
     def getAudioCaps(self):
         """ Returns the GstCaps corresponding to the audio settings """
         astr = "rate=%d,channels=%d" % (self.audiorate, self.audiochannels)
@@ -542,7 +548,8 @@ class ExportSettings(Signallable, Loggable):
 
         # interset with current audioencoder sink pad caps
 
-    def setVideoProperties(self, width=-1, height=-1, framerate=-1, par=-1):
+    def setVideoProperties(self, width=-1, height=-1, framerate=-1, par=-1,
+            render_scale=-1):
         """ Set the video width, height and framerate """
         self.info("set_video_props %d x %d @ %r fps", width, height, framerate)
         changed = False
@@ -552,6 +559,9 @@ class ExportSettings(Signallable, Loggable):
         if not height == -1 and not height == self.videoheight:
             self.videoheight = height
             changed = True
+        if not render_scale == -1 and not render_scale == self.render_scale:
+            self.render_scale = render_scale
+            changed = True
         if not framerate == -1 and not framerate == self.videorate:
             self.videorate = framerate
             changed = True
@@ -592,11 +602,47 @@ class ExportSettings(Signallable, Loggable):
         if changed:
             self.emit("encoders-changed")
 
+    @property
+    def containersettings(self):
+        return self._containersettings_cache.setdefault(self.muxer, {})
+
+    @containersettings.setter
+    def containersettings(self, value):
+        self._containersettings_cache[self.muxer] = value
+
+    @property
+    def vcodecsettings(self):
+        return self._vcodecsettings_cache.setdefault(self.vencoder, {})
+
+    @vcodecsettings.setter
+    def vcodecsettings(self, value):
+        self._vcodecsettings_cache[self.vencoder] = value
+
+    @property
+    def acodecsettings(self):
+        return self._acodecsettings_cache.setdefault(self.aencoder, {})
+
+    @acodecsettings.setter
+    def acodecsettings(self, value):
+        self._acodecsettings_cache[self.aencoder] = value
+
+    def getAudioEncoders(self):
+        """ Returns the list of audio encoders compatible with the current
+        muxer """
+        return self.aencoders[self.muxer]
+
+    def getVideoEncoders(self):
+        """ Returns the list of video encoders compatible with the current
+        muxer """
+        return self.vencoders[self.muxer]
+
 def export_settings_to_render_settings(export,
         have_video=True, have_audio=True):
+    """Convert the specified ExportSettings object to a RenderSettings object.
+    """
     # Get the audio and video caps/encoder/settings
     astream = get_stream_for_caps(export.getAudioCaps())
-    vstream = get_stream_for_caps(export.getVideoCaps())
+    vstream = get_stream_for_caps(export.getVideoCaps(render=True))
 
     encoder_settings = []
     if export.vencoder is not None and have_video:
