@@ -29,8 +29,7 @@ import os
 import sys
 
 import gtk
-import gtk.glade
-import gobject
+from gtk.glade import XML, set_custom_handler
 
 # proc := module1.module2.moduleN.proc1().maybe_another_proc()
 #  -> eval proc1().maybe_another_proc() in module1.module2.moduleN
@@ -63,7 +62,7 @@ def flumotion_glade_custom_handler(unused_xml, proc, name, *unused_args):
     w.set_name(name)
     w.show()
     return w
-gtk.glade.set_custom_handler(flumotion_glade_custom_handler)
+set_custom_handler(flumotion_glade_custom_handler)
 
 
 class GladeWidget(gtk.VBox):
@@ -86,12 +85,16 @@ class GladeWidget(gtk.VBox):
         gtk.VBox.__init__(self)
         try:
             assert self.glade_file
-            filepath = os.path.join(self.glade_dir, self.glade_file)
+            if 'pitivi.exe' in __file__.lower():
+                from pitivi.configure import WIN32_LIBDIR
+                filepath = os.path.join(self.glade_dir, WIN32_LIBDIR + self.glade_file)
+            else:
+                filepath = os.path.join(self.glade_dir, self.glade_file)
             if self.glade_typedict:
-                wtree = gtk.glade.XML(filepath, typedict=self.glade_typedict, domain='pitivi')
+                wtree = XML(filepath, typedict=self.glade_typedict, domain='pitivi')
             else:
                 # pygtk 2.4 doesn't like typedict={} ?
-                wtree = gtk.glade.XML(filepath, domain='pitivi')
+                wtree = XML(filepath, domain='pitivi')
         except RuntimeError, e:
             raise RuntimeError('Failed to load file %s from directory %s: %s'
                                % (self.glade_file, self.glade_dir, e))
@@ -118,7 +121,7 @@ class GladeWidget(gtk.VBox):
         wtree.signal_autoconnect(self)
 
 
-class GladeWindow(gobject.GObject):
+class GladeWindow(object):
     """
     Base class for dialogs or windows backed by glade interface definitions.
 
@@ -140,15 +143,18 @@ class GladeWindow(gobject.GObject):
     window = None
 
     def __init__(self, parent=None):
-        gobject.GObject.__init__(self)
         try:
             assert self.glade_file
-            filepath = os.path.join(self.glade_dir, self.glade_file)
+            if 'pitivi.exe' in __file__.lower():
+                from pitivi.configure import WIN32_LIBDIR
+                filepath = os.path.join(self.glade_dir, WIN32_LIBDIR + self.glade_file)
+            else:
+                filepath = os.path.join(self.glade_dir, self.glade_file)
             if self.glade_typedict:
-                wtree = gtk.glade.XML(filepath, typedict=self.glade_typedict, domain='pitivi')
+                wtree = XML(filepath, typedict=self.glade_typedict, domain='pitivi')
             else:
                 # pygtk 2.4 doesn't like typedict={} ?
-                wtree = gtk.glade.XML(filepath, domain='pitivi')
+                wtree = XML(filepath, domain='pitivi')
         except RuntimeError, e:
             raise RuntimeError('Failed to load file %s from directory %s: %s'
                                % (self.glade_file, self.glade_dir, e))
@@ -165,6 +171,7 @@ class GladeWindow(gobject.GObject):
                 raise AssertionError("Two objects with same name (%s): %r %r"
                                      % (wname, self.widgets[wname], widget))
             self.widgets[wname] = widget
+            setattr(self, wname, widget)
 
         if parent:
             self.window.set_transient_for(parent)
