@@ -13,7 +13,7 @@ import controller
 from zoominterface import Zoomable
 from pitivi.timeline.track import TrackError
 from pitivi.timeline.timeline import SELECT, SELECT_ADD, UNSELECT, \
-    MoveContext, TrimStartContext, TrimEndContext
+    SELECT_BETWEEN, MoveContext, TrimStartContext, TrimEndContext
 from preview import Preview
 from pitivi.ui.curve import Curve
 import gst
@@ -41,7 +41,7 @@ import gst
 GlobalSettings.addConfigOption('videoClipBg',
     section = 'user-interface',
     key = 'videoclip-background',
-    default = 0x3182bdC0,
+    default = 0x000000A0,
     notify = True)
 
 PreferencesDialog.addColorPreference('videoClipBg',
@@ -52,7 +52,7 @@ PreferencesDialog.addColorPreference('videoClipBg',
 GlobalSettings.addConfigOption('audioClipBg',
     section = 'user-interface',
     key = 'audioclip-background',
-    default = 0x3182bdC0,
+    default = 0x4E9A06C0,
     notify = True)
 
 PreferencesDialog.addColorPreference('audioClipBg',
@@ -97,6 +97,7 @@ class TimelineController(controller.Controller):
 
     _cursor = ARROW
     _context = None
+    _handle_enter_leave = False
 
     def enter(self, unused, unused2):
         self._view.focus()
@@ -120,8 +121,9 @@ class TimelineController(controller.Controller):
 
     def set_pos(self, item, pos):
         x, y = pos
-        position = Zoomable.pixelToNs(x)
-        priority = int((y - self._y_offset) // (LAYER_HEIGHT_EXPANDED + LAYER_SPACING))
+        position = Zoomable.pixelToNs(x + self._hadj.get_value())
+        priority = int((y - self._y_offset + self._vadj.get_value()) //
+            (LAYER_HEIGHT_EXPANDED + LAYER_SPACING))
         self._context.setMode(self._getMode())
         self._context.editTo(position, priority)
 
@@ -154,6 +156,7 @@ class TrimHandle(View, goocanvas.Image, Zoomable):
         goocanvas.Image.__init__(self,
             pixbuf = TRIMBAR_PIXBUF,
             line_width=0,
+            pointer_events=goocanvas.EVENTS_FILL,
             **kwargs
         )
         View.__init__(self)
@@ -197,12 +200,12 @@ class EndHandle(TrimHandle):
 
 class TrackObject(View, goocanvas.Group, Zoomable):
 
-
     class Controller(TimelineController):
 
+        _handle_enter_leave = True
+
         def drag_start(self, item, target, event):
             TimelineController.drag_start(self, item, target, event)
-            self._view.raise_(None)
             self._context = MoveContext(self._view.timeline,
                 self._view.element,
                 self._view.timeline.selection.getSelectedTrackObjs())
@@ -214,13 +217,22 @@ class TrackObject(View, goocanvas.Group, Zoomable):
             return self._context.DEFAULT
 
         def click(self, pos):
-            mode = SELECT
+            timeline = self._view.timeline
+            element = self._view.element
+            element_end = element.start + element.duration
             if self._last_event.get_state() & gtk.gdk.SHIFT_MASK:
-                mode = SELECT_ADD
+                timeline.setSelectionToObj(element, SELECT_BETWEEN)
             elif self._last_event.get_state() & gtk.gdk.CONTROL_MASK:
-                mode = UNSELECT
-            self._view.timeline.setSelectionToObj(
-                self._view.element, mode)
+                if element.selected:
+                    mode = UNSELECT
+                else:
+                    mode = SELECT_ADD
+                timeline.setSelectionToObj(element, mode)
+            else:
+                x, y = pos
+                x += self._hadj.get_value()
+                self._view.app.current.seeker.seek(Zoomable.pixelToNs(x))
+                timeline.setSelectionToObj(element, SELECT)
 
     def __init__(self, instance, element, track, timeline):
         goocanvas.Group.__init__(self)
@@ -233,10 +245,10 @@ class TrackObject(View, goocanvas.Group, Zoomable):
         self.nameheight = 0
 
         self.bg = goocanvas.Rect(
-            height=self.height, 
-            line_width=0)
+            height=self.height,
+            line_width=1)
 
-        self.content = Preview(element)
+        self.content = Preview(self.app, element)
 
         self.name = goocanvas.Text(
             x= NAME_HOFFSET + NAME_PADDING,
@@ -260,16 +272,16 @@ class TrackObject(View, goocanvas.Group, Zoomable):
             line_width = 0.0,
             height=self.height)
 
-        for thing in (self.bg, self.content, self.selection_indicator, 
+        for thing in (self.bg, self.content, self.selection_indicator,
             self.start_handle, self.end_handle, self.namebg, self.name):
             self.add_child(thing)
 
         for prop, interpolator in element.getInterpolators().itervalues():
-            self.add_child(Curve(instance, element, interpolator, 50))
+            self.add_child(Curve(instance, element, interpolator))
 
         self.element = element
         self.settings = instance.settings
-        self.normal()
+        self.unfocus()
 
 ## Properties
 
@@ -312,12 +324,13 @@ class TrackObject(View, goocanvas.Group, Zoomable):
 ## Public API
 
     def focus(self):
-        self.start_handle.focus()
-        self.end_handle.focus()
+        self.start_handle.props.visibility = goocanvas.ITEM_VISIBLE
+        self.end_handle.props.visibility = goocanvas.ITEM_VISIBLE
+        self.raise_(None)
 
     def unfocus(self):
-        self.start_handle.unfocus()
-        self.end_handle.unfocus()
+        self.start_handle.props.visibility = goocanvas.ITEM_INVISIBLE
+        self.end_handle.props.visibility = goocanvas.ITEM_INVISIBLE
 
     def zoomChanged(self):
         self._update()
@@ -394,6 +407,9 @@ class TrackObject(View, goocanvas.Group, Zoomable):
         y = (self.height + LAYER_SPACING) * self.element.priority
         self.set_simple_transform(x, y, 1, 0)
         width = self.nsToPixel(self.element.duration)
+        min_width = self.start_handle.props.width * 2
+        if width < min_width:
+            width = min_width
         w = width - self.end_handle.props.width
         self.name.props.clip_path = "M%g,%g h%g v%g h-%g z" % (
             0, 0, w, self.height, w)
@@ -403,7 +419,7 @@ class TrackObject(View, goocanvas.Group, Zoomable):
         if self.expanded:
             if w - NAME_HOFFSET > 0:
                 self.namebg.props.height = self.nameheight + NAME_PADDING2X
-                self.namebg.props.width = min(w - NAME_HOFFSET, 
+                self.namebg.props.width = min(w - NAME_HOFFSET,
                     self.namewidth + NAME_PADDING2X)
                 self.namebg.props.visibility = goocanvas.ITEM_VISIBLE
             else:
