@@ -20,17 +20,45 @@
 # Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 # Boston, MA 02111-1307, USA.
 
+import platform
 import gobject
 import gtk
 from gtk import gdk
 import gst
 
+from gettext import gettext as _
+
 from pitivi.action import ViewAction
 
 from pitivi.stream import VideoStream
 from pitivi.utils import time_to_string, Seeker
 from pitivi.log.loggable import Loggable
 from pitivi.pipeline import PipelineError
+from pitivi.ui.common import SPACING
+from pitivi.settings import GlobalSettings
+from pitivi.ui.dynamic import TimeWidget
+
+GlobalSettings.addConfigSection("viewer")
+GlobalSettings.addConfigOption("viewerDocked",
+    section="viewer",
+    key="docked",
+    default=True)
+GlobalSettings.addConfigOption("viewerWidth",
+    section="viewer",
+    key="width",
+    default=320)
+GlobalSettings.addConfigOption("viewerHeight",
+    section="viewer",
+    key="height",
+    default=240)
+GlobalSettings.addConfigOption("viewerX",
+    section="viewer",
+    key="x-pos",
+    default=0)
+GlobalSettings.addConfigOption("viewerY",
+    section="viewer",
+    key="y-pos",
+    default=0)
 
 class ViewerError(Exception):
     pass
@@ -41,7 +69,7 @@ class PitiviViewer(gtk.VBox, Loggable):
 
     __gtype_name__ = 'PitiviViewer'
     __gsignals__ = {
-        "activate-playback-controls" : (gobject.SIGNAL_RUN_LAST, 
+        "activate-playback-controls" : (gobject.SIGNAL_RUN_LAST,
             gobject.TYPE_NONE, (gobject.TYPE_BOOLEAN,)),
     }
 
@@ -54,12 +82,17 @@ class PitiviViewer(gtk.VBox, Loggable):
     @type action: L{ViewAction}
     """
 
-    def __init__(self, action=None, pipeline=None):
+    def __init__(self, app, undock_action=None, action=None,
+                 pipeline=None):
         """
         @param action: Specific action to use instead of auto-created one
         @type action: L{ViewAction}
         """
         gtk.VBox.__init__(self)
+        self.set_border_width(SPACING)
+        self.settings = app.settings
+        self.app = app
+
         Loggable.__init__(self)
         self.log("New PitiviViewer")
 
@@ -67,6 +100,8 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.seeker.connect('seek', self._seekerSeekCb)
         self.action = action
         self.pipeline = pipeline
+        self.sink = None
+        self.docked = True
 
         self.current_time = long(0)
         self._initial_seek = None
@@ -76,8 +111,15 @@ class PitiviViewer(gtk.VBox, Loggable):
         self._haveUI = False
 
         self._createUi()
+        self.target = self.internal
         self.setAction(action)
         self.setPipeline(pipeline)
+        self.undock_action = undock_action
+        if undock_action:
+            self.undock_action.connect("activate", self._toggleDocked)
+
+            if not self.settings.viewerDocked:
+                self.undock()
 
     def setPipeline(self, pipeline):
         """
@@ -159,8 +201,10 @@ class PitiviViewer(gtk.VBox, Loggable):
             self.pipeline.removeAction(self.action)
 
         self.pipeline.disconnect_by_function(self._posCb)
+        self.pipeline.disconnect_by_function(self._currentStateCb)
         self.pipeline.disconnect_by_function(self._elementMessageCb)
-        #self.deactivatePositionListener()
+        self.pipeline.disconnect_by_function(self._durationChangedCb)
+        self.pipeline.disconnect_by_function(self._eosCb)
         self.pipeline.stop()
 
         self.pipeline = None
@@ -170,7 +214,7 @@ class PitiviViewer(gtk.VBox, Loggable):
         # not sure what we need to do ...
         self.action = action
         # FIXME: fix this properly?
-        self.drawingarea.action = action
+        self.internal.action = action
         dar = float(4/3)
         try:
             producer = action.producers[0]
@@ -194,9 +238,9 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.debug("active %r", active)
         self.set_sensitive(active)
         if self._haveUI:
-            for item in [self.slider, self.rewind_button, self.back_button,
-                         self.playpause_button, self.next_button,
-                         self.forward_button, self.timelabel]:
+            for item in [self.slider, self.goToStart_button, self.back_button,
+                         self.playpause_button, self.forward_button,
+                         self.goToEnd_button, self.timecode_entry]:
                 item.set_sensitive(active)
         if active:
             self.emit("activate-playback-controls", True)
@@ -204,14 +248,38 @@ class PitiviViewer(gtk.VBox, Loggable):
     def _getDefaultAction(self):
         return ViewAction()
 
+    def _externalWindowDeleteCb(self, window, event):
+        self.dock()
+        return True
+
+    def _externalWindowConfigureCb(self, window, event):
+        self.settings.viewerWidth = event.width
+        self.settings.viewerHeight = event.height
+        self.settings.viewerX = event.x
+        self.settings.viewerY = event.y
+
     def _createUi(self):
         """ Creates the Viewer GUI """
         # drawing area
         self.aframe = gtk.AspectFrame(xalign=0.5, yalign=0.5, ratio=4.0/3.0,
                                       obey_child=False)
         self.pack_start(self.aframe, expand=True)
-        self.drawingarea = ViewerWidget(self.action)
-        self.aframe.add(self.drawingarea)
+        self.internal = ViewerWidget(self.action)
+        self.internal.show()
+        self.aframe.add(self.internal)
+
+        self.external_window = gtk.Window()
+        vbox = gtk.VBox()
+        vbox.set_spacing(SPACING)
+        self.external_window.add(vbox)
+        self.external = ViewerWidget(self.action)
+        vbox.pack_start(self.external)
+        self.external_window.connect("delete-event",
+            self._externalWindowDeleteCb)
+        self.external_window.connect("configure-event",
+            self._externalWindowConfigureCb)
+        self.external_vbox = vbox
+        self.external_vbox.show_all()
 
         # Slider
         self.posadjust = gtk.Adjustment()
@@ -230,13 +298,15 @@ class PitiviViewer(gtk.VBox, Loggable):
         boxalign.add(bbox)
         self.pack_start(boxalign, expand=False)
 
-        self.rewind_button = gtk.ToolButton(gtk.STOCK_MEDIA_REWIND)
-        self.rewind_button.connect("clicked", self._rewindCb)
-        self.rewind_button.set_sensitive(False)
-        bbox.pack_start(self.rewind_button, expand=False)
+        self.goToStart_button = gtk.ToolButton(gtk.STOCK_MEDIA_PREVIOUS)
+        self.goToStart_button.connect("clicked", self._goToStartCb)
+        self.goToStart_button.set_tooltip_text(_("Go to the beginning of the timeline"))
+        self.goToStart_button.set_sensitive(False)
+        bbox.pack_start(self.goToStart_button, expand=False)
 
-        self.back_button = gtk.ToolButton(gtk.STOCK_MEDIA_PREVIOUS)
+        self.back_button = gtk.ToolButton(gtk.STOCK_MEDIA_REWIND)
         self.back_button.connect("clicked", self._backCb)
+        self.back_button.set_tooltip_text(_("Go back one second"))
         self.back_button.set_sensitive(False)
         bbox.pack_start(self.back_button, expand=False)
 
@@ -245,21 +315,24 @@ class PitiviViewer(gtk.VBox, Loggable):
         bbox.pack_start(self.playpause_button, expand=False)
         self.playpause_button.set_sensitive(False)
 
-        self.next_button = gtk.ToolButton(gtk.STOCK_MEDIA_NEXT)
-        self.next_button.connect("clicked", self._nextCb)
-        self.next_button.set_sensitive(False)
-        bbox.pack_start(self.next_button, expand=False)
-
         self.forward_button = gtk.ToolButton(gtk.STOCK_MEDIA_FORWARD)
         self.forward_button.connect("clicked", self._forwardCb)
+        self.forward_button.set_tooltip_text(_("Go forward one second"))
         self.forward_button.set_sensitive(False)
         bbox.pack_start(self.forward_button, expand=False)
 
+        self.goToEnd_button = gtk.ToolButton(gtk.STOCK_MEDIA_NEXT)
+        self.goToEnd_button.connect("clicked", self._goToEndCb)
+        self.goToEnd_button.set_tooltip_text(_("Go to the end of the timeline"))
+        self.goToEnd_button.set_sensitive(False)
+        bbox.pack_start(self.goToEnd_button, expand=False)
+
         # current time
-        self.timelabel = gtk.Label()
-        self.timelabel.set_markup("<tt>00:00:00.000</tt>")
-        self.timelabel.set_alignment(1.0, 0.5)
-        bbox.pack_start(self.timelabel, expand=False, padding=10)
+        self.timecode_entry = TimeWidget()
+        self.timecode_entry.setWidgetValue(0)
+        self.timecode_entry.connect("value-changed", self._jumpToTimecodeCb)
+        self.timecode_entry.connectFocusEvents(self._entryFocusInCb, self._entryFocusOutCb)
+        bbox.pack_start(self.timecode_entry, expand=False, padding=10)
         self._haveUI = True
 
         screen = gdk.screen_get_default()
@@ -273,6 +346,7 @@ class PitiviViewer(gtk.VBox, Loggable):
             height = int(width / self.aframe.props.ratio)
             self.aframe.set_size_request(width , height)
         self.show_all()
+        self.buttons = boxalign
 
     _showingSlider = True
 
@@ -288,19 +362,19 @@ class PitiviViewer(gtk.VBox, Loggable):
         if not self.action:
             return
         if True:
-            self.rewind_button.show()
+            self.goToStart_button.show()
             self.back_button.show()
             self.playpause_button.show()
-            self.next_button.show()
             self.forward_button.show()
+            self.goToEnd_button.show()
             if self._showingSlider:
                 self.slider.show()
         else:
-            self.rewind_button.hide()
+            self.goToStart_button.hide()
             self.back_button.hide()
             self.playpause_button.hide()
-            self.next_button.hide()
             self.forward_button.hide()
+            self.goToEnd_button.hide()
             self.slider.hide()
 
     def setDisplayAspectRatio(self, ratio):
@@ -318,6 +392,16 @@ class PitiviViewer(gtk.VBox, Loggable):
 
     ## gtk.HScale callbacks for self.slider
 
+    def _entryFocusInCb(self, entry, event):
+        sensitive_actions = self.app.gui.sensitive_actions
+        self.app.gui.setActionsSensitive(sensitive_actions, False)
+        self.app.gui.setActionsSensitive(['DeleteObj'], False)
+
+    def _entryFocusOutCb(self, entry, event):
+        sensitive_actions = self.app.gui.sensitive_actions
+        self.app.gui.setActionsSensitive(sensitive_actions, True)
+        self.app.gui.setActionsSensitive(['DeleteObj'], True)
+
     def _sliderButtonPressCb(self, slider, event):
         # borrow totem hack for seek-on-click behavior
         event.button = 2
@@ -372,7 +456,7 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.info("value:%s, frame:%d", gst.TIME_ARGS(value), frame)
         self.current_time = value
         self.current_frame = frame
-        self.timelabel.set_markup("<tt>%s</tt>" % time_to_string(value))
+        self.timecode_entry.setWidgetValue(value, False)
         if not self.moving_slider:
             self.posadjust.set_value(float(value))
         return False
@@ -398,7 +482,7 @@ class PitiviViewer(gtk.VBox, Loggable):
 
     ## Control gtk.Button callbacks
 
-    def _rewindCb(self, unused_button):
+    def _goToStartCb(self, unused_button):
         self.seek(0)
 
     def _backCb(self, unused_button):
@@ -407,16 +491,22 @@ class PitiviViewer(gtk.VBox, Loggable):
     def _playButtonCb(self, unused_button, isplaying):
         self.togglePlayback()
 
-    def _nextCb(self, unused_button):
+    def _forwardCb(self, unused_button):
         self.seekRelative(gst.SECOND)
 
-    def _forwardCb(self, unused_button):
+    def _goToEndCb(self, unused_button):
         try:
             dur = self.pipeline.getDuration()
             self.seek(dur - 1)
         except:
             self.warning("couldn't get duration")
 
+    ## Callback for jumping to a specific timecode
+
+    def _jumpToTimecodeCb(self, widget):
+        nanoseconds = widget.getWidgetValue()
+        self.seek(nanoseconds)
+
     ## public methods for controlling playback
 
     def play(self):
@@ -430,6 +520,60 @@ class PitiviViewer(gtk.VBox, Loggable):
             return
         self.pipeline.togglePlayback()
 
+    def undock(self):
+        if not self.undock_action:
+            self.error("Cannot undock because undock_action is missing.")
+            return
+        if not self.docked:
+            return
+
+        self.docked = False
+        self.settings.viewerDocked = False
+        self.undock_action.set_label(_("Dock Viewer"))
+
+        self.remove(self.buttons)
+        self.remove(self.slider)
+        self.external_vbox.pack_end(self.slider, False, False)
+        self.external_vbox.pack_end(self.buttons, False, False)
+        self.external_window.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_UTILITY)
+        self.external_window.show()
+        self.target = self.external
+        # if we are playing, switch output immediately
+        if self.sink:
+            self._switch_output_window()
+        self.hide()
+        self.external_window.move(self.settings.viewerX,
+            self.settings.viewerY)
+        self.external_window.resize(self.settings.viewerWidth,
+            self.settings.viewerHeight)
+
+    def dock(self):
+        if not self.undock_action:
+            self.error("Cannot dock because undock_action is missing.")
+            return
+        if self.docked:
+            return
+        self.docked = True
+        self.settings.viewerDocked = True
+        self.undock_action.set_label(_("Undock Viewer"))
+
+        self.target = self.internal
+        self.external_vbox.remove(self.slider)
+        self.external_vbox.remove(self.buttons)
+        self.pack_end(self.slider, False, False)
+        self.pack_end(self.buttons, False, False)
+        self.show()
+        # if we are playing, switch output immediately
+        if self.sink:
+            self._switch_output_window()
+        self.external_window.hide()
+
+    def _toggleDocked(self, action):
+        if self.docked:
+            self.undock()
+        else:
+            self.dock()
+
     def seekRelative(self, time):
         try:
             self.pipeline.seekRelative(time)
@@ -445,6 +589,8 @@ class PitiviViewer(gtk.VBox, Loggable):
             self.playpause_button.setPause()
         elif state == int(gst.STATE_PAUSED):
             self.playpause_button.setPlay()
+        else:
+            self.sink = None
         self.currentState = state
 
     def _eosCb(self, unused_pipeline):
@@ -455,7 +601,14 @@ class PitiviViewer(gtk.VBox, Loggable):
         self.log('message:%s / %s', message, name)
         if name == 'prepare-xwindow-id':
             sink = message.src
-            sink.set_xwindow_id(self.drawingarea.window_xid)
+            self.sink = sink
+            self._switch_output_window()
+
+    def _switch_output_window(self):
+        gtk.gdk.threads_enter()
+        self.sink.set_xwindow_id(self.target.window_xid)
+        self.sink.expose()
+        gtk.gdk.threads_leave()
 
 
 class ViewerWidget(gtk.DrawingArea, Loggable):
@@ -475,7 +628,10 @@ class ViewerWidget(gtk.DrawingArea, Loggable):
 
     def do_realize(self):
         gtk.DrawingArea.do_realize(self)
-        self.window_xid = self.window.xid
+        if platform.system() == 'Windows':
+            self.window_xid = self.window.handle
+        else:
+            self.window_xid  = self.window.xid
 
 class PlayPauseButton(gtk.Button, Loggable):
     """ Double state gtk.Button which displays play/pause """
@@ -505,6 +661,7 @@ class PlayPauseButton(gtk.Button, Loggable):
         self.log("setPlay")
         if self.playing:
             self.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PLAY, gtk.ICON_SIZE_BUTTON))
+            self.set_tooltip_text(_("Play"))
             self.playing = False
 
     def setPause(self):
@@ -512,4 +669,5 @@ class PlayPauseButton(gtk.Button, Loggable):
         """ display the pause image """
         if not self.playing:
             self.set_image(gtk.image_new_from_stock(gtk.STOCK_MEDIA_PAUSE, gtk.ICON_SIZE_BUTTON))
+            self.set_tooltip_text(_("Pause"))
             self.playing = True
